@@ -49,7 +49,7 @@ class ErrorMiddleware(object):
           If true, then errors will be printed to ``wsgi.errors`` 
           (frequently a server error log, or stderr).
 
-      ``from_address``, ``smtp_server``, ``error_subject_prefix``:
+      ``from_address``, ``smtp_server``, ``error_subject_prefix``, ``smtp_username``, ``smtp_password``, ``smtp_use_tls``:
           variables to control the emailed exception reports
 
       ``error_message``:
@@ -87,6 +87,9 @@ class ErrorMiddleware(object):
                  show_exceptions_in_wsgi_errors=NoDefault,
                  from_address=None,
                  smtp_server=None,
+                 smtp_username=None,
+                 smtp_password=None,
+                 smtp_use_tls=False,
                  error_subject_prefix=None,
                  error_message=None,
                  xmlhttp_key=None,
@@ -111,11 +114,19 @@ class ErrorMiddleware(object):
         self.error_log = error_log
         self.show_exceptions_in_wsgi_errors = show_exceptions_in_wsgi_errors
         if from_address is None:
-            from_address = global_conf.get('error_from_address', 'errors@localhost')
+            from_address = global_conf.get('error_from_address')
+            if from_address is None:
+                if self.error_email:
+                    from_address = self.error_email[0]
+                else:
+                    from_address = 'errors@localhost'
         self.from_address = from_address
         if smtp_server is None:
             smtp_server = global_conf.get('smtp_server', 'localhost')
         self.smtp_server = smtp_server
+        self.smtp_username = smtp_username or global_conf.get('smtp_username')
+        self.smtp_password = smtp_password or global_conf.get('smtp_password')
+        self.smtp_use_tls = smtp_use_tls or converters.asbool(global_conf.get('smtp_use_tls'))
         self.error_subject_prefix = error_subject_prefix or ''
         if error_message is None:
             error_message = global_conf.get('error_message')
@@ -147,26 +158,29 @@ class ErrorMiddleware(object):
 
         try:
             __traceback_supplement__ = Supplement, self, environ
-            app_iter = self.application(environ, start_response)
-            return self.make_catching_iter(app_iter, environ)
+            sr_checker = ResponseStartChecker(start_response)
+            app_iter = self.application(environ, sr_checker)
+            return self.make_catching_iter(app_iter, environ, sr_checker)
         except:
             exc_info = sys.exc_info()
             try:
                 start_response('500 Internal Server Error',
-                               [('content-type', 'text/html')],
+                               [('content-type', 'text/html; charset=utf8')],
                                exc_info)
                 # @@: it would be nice to deal with bad content types here
                 response = self.exception_handler(exc_info, environ)
+                if isinstance(response, unicode):
+                    response = response.encode('utf8')
                 return [response]
             finally:
                 # clean up locals...
                 exc_info = None
 
-    def make_catching_iter(self, app_iter, environ):
+    def make_catching_iter(self, app_iter, environ, sr_checker):
         if isinstance(app_iter, (list, tuple)):
             # These don't raise
             return app_iter
-        return CatchingIter(app_iter, environ, self)
+        return CatchingIter(app_iter, environ, sr_checker, self)
 
     def exception_handler(self, exc_info, environ):
         simple_html_error = False
@@ -183,11 +197,23 @@ class ErrorMiddleware(object):
             show_exceptions_in_wsgi_errors=self.show_exceptions_in_wsgi_errors,
             error_email_from=self.from_address,
             smtp_server=self.smtp_server,
+            smtp_username=self.smtp_username,
+            smtp_password=self.smtp_password,
+            smtp_use_tls=self.smtp_use_tls,
             error_subject_prefix=self.error_subject_prefix,
             error_message=self.error_message,
             simple_html_error=simple_html_error,
             reporters=self.reporters)
 
+class ResponseStartChecker(object):
+    def __init__(self, start_response):
+        self.start_response = start_response
+        self.response_started = False
+
+    def __call__(self, *args):
+        self.response_started = True
+        self.start_response(*args)
+
 class CatchingIter(object):
 
     """
@@ -196,10 +222,11 @@ class CatchingIter(object):
     display or report as necessary.
     """
 
-    def __init__(self, app_iter, environ, error_middleware):
+    def __init__(self, app_iter, environ, start_checker, error_middleware):
         self.app_iterable = app_iter
         self.app_iterator = iter(app_iter)
         self.environ = environ
+        self.start_checker = start_checker
         self.error_middleware = error_middleware
         self.closed = False
 
@@ -223,12 +250,19 @@ class CatchingIter(object):
         except:
             self.closed = True
             close_response = self._close()
+            exc_info = sys.exc_info()
             response = self.error_middleware.exception_handler(
-                sys.exc_info(), self.environ)
+                exc_info, self.environ)
             if close_response is not None:
                 response += (
                     '<hr noshade>Error in .close():<br>%s'
                     % close_response)
+
+            if not self.start_checker.response_started:
+                self.start_checker('500 Internal Server Error',
+                               [('content-type', 'text/html')],
+                               exc_info)
+
             return response
 
     def close(self):
@@ -309,6 +343,9 @@ def handle_exception(exc_info, error_stream, html=True,
                      show_exceptions_in_wsgi_errors=False,
                      error_email_from='errors@localhost',
                      smtp_server='localhost',
+                     smtp_username=None, 
+                     smtp_password=None, 
+                     smtp_use_tls=False,
                      error_subject_prefix='',
                      error_message=None,
                      simple_html_error=False,
@@ -340,6 +377,9 @@ def handle_exception(exc_info, error_stream, html=True,
             to_addresses=error_email,
             from_address=error_email_from,
             smtp_server=smtp_server,
+            smtp_username=smtp_username,
+            smtp_password=smtp_password,
+            smtp_use_tls=smtp_use_tls,
             subject_prefix=error_subject_prefix)
         rep_err = send_report(rep, exc_data, html=html)
         if rep_err:
@@ -442,7 +482,7 @@ def error_template(head_html, exception, extra):
 def make_error_middleware(app, global_conf, **kw):
     return ErrorMiddleware(app, global_conf=global_conf, **kw)
 
-doc_lines = ErrorMiddleware.__doc__.splitlines(True)
+doc_lines = (ErrorMiddleware.__doc__ or '').splitlines(True)
 for i in range(len(doc_lines)):
     if doc_lines[i].strip().startswith('Settings'):
         make_error_middleware.__doc__ = ''.join(doc_lines[i:])
